#
# network_text.py: text mode network configuration dialogs
#
# Copyright 2001 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# library public license.
#
# You should have received a copy of the GNU Library Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import iutil
import os
import isys
from snack import *
from constants_text import *
from translate import _

class NetworkWindow:
    def setsensitive (self):
        if self.cb.selected ():
            sense = FLAGS_SET
        else:
            sense = FLAGS_RESET

        for n in self.ip, self.nm:
            n.setFlags (FLAG_DISABLED, sense)

    def calcNM (self):
        ip = self.ip.value ()
        if ip and not self.nm.value ():
            try:
                mask = isys.inet_calcNetmask (ip)
            except ValueError:
                return

            self.nm.set (mask)

    def calcGW (self):

	# suggest a gateway and nameserver when host has only one NIC
	if self.numdevices > 1:
	    return

        ip = self.ip.value ()
        nm = self.nm.value ()
        if ip and nm:
            try:
                (net, bcast) = isys.inet_calcNetBroad (ip, nm)
            except ValueError:
                return

            if not self.gw.value ():
                gw = isys.inet_calcGateway (net)
                self.gw.set (gw)
            if not self.ns.value ():
                ns = isys.inet_calcNS (net)
                self.ns.set (ns)


    # Configure one network device
    def configureDevice (self, screen, network, device):

	buttons = ButtonBar (screen, (("Ok", "ok"), ("Skip", "skip"), ("Back", "back")))

	text = TextboxReflowed (35, "Please enter network information for device " + device.get("device") + ".")

        firstg = Grid (1, 2)
        boot = device.get ("bootproto")
        onboot = device.get('onboot')
        onbootIsOn = ((device == network.available().values()[0] and not onboot)
                      or onboot == 'yes')
        if not boot:
            boot = "static"

        self.cb = Checkbox (_("Use bootp/dhcp"),
                            isOn = (boot == "dhcp"))
        firstg.setField (self.cb, 0, 0, anchorLeft = 1)
        self.onboot = Checkbox(_("Activate on boot"), isOn = onbootIsOn)
        firstg.setField (self.onboot, 0, 1, anchorLeft = 1)        

	grid = Grid (2, 2)
	grid.setField (Label ("IP address:"), 0, 0, anchorLeft = 1)
	grid.setField (Label ("Netmask:"), 0, 1, anchorLeft = 1)

	self.ip = Entry (16)
	self.nm = Entry (16)
	self.ip.set (device.get ("ipaddr"))
	self.nm.set (device.get ("netmask"))

	grid.setField (self.ip, 1, 0, (1, 0, 0, 0))
	grid.setField (self.nm, 1, 1, (1, 0, 0, 0))


	toplevel = GridForm (screen, "Network configuration "
                             + device.get("device"), 1, 4)
	toplevel.add (text, 0, 0, (0, 0, 0, 1), anchorLeft = 1)
        toplevel.add (firstg, 0, 1, (0, 0, 0, 1))
	toplevel.add (grid, 0, 2, (0, 0, 0, 1))
	toplevel.add (buttons, 0, 3, growx = 1)

        self.cb.setCallback (self.setsensitive)
	self.ip.setCallback (self.calcNM)
	#self.nm.setCallback (self.calcGW)

        self.setsensitive()
	while 1:
	    result = toplevel.run()

	    if buttons.buttonPressed (result) == "back":
		screen.popWindow()
		return INSTALL_BACK
	    if buttons.buttonPressed (result) == "skip":
		# If the device has not been configured before, don't make
		# it start automatically
		if not device.get ("bootproto"):
		    device.set (("onboot", "no"))
		screen.popWindow()
		return "skip"

            if self.onboot.selected() != 0:
                device.set (('onboot', 'yes'))
            else:
                device.unset ('onboot')

            if self.cb.selected ():
                device.set (("bootproto", "dhcp"))
                device.unset ("ipaddr", "netmask", "network", "broadcast")
            else:           
                try:
                    (network, broadcast) = isys.inet_calcNetBroad (self.ip.value (), self.nm.value ())
                except:
                    ButtonChoiceWindow (screen, _("Invalid information"),
				    _("You must enter valid IP information to continue"),
				    buttons = [ _("OK") ])
                    continue

                device.set(("bootproto", "static"))
                device.set(("ipaddr", self.ip.value()), ("netmask", self.nm.value()),
                           ("network", network), ("broadcast", broadcast))
                device.set (("onboot", "yes"))

	        #todo.log ("\"" + device.get ("device") + "\"")

	    screen.popWindow()
	    return INSTALL_OK

    def configureNetwork (self, screen, network):
	text = TextboxReflowed (35, "Please enter your default gateway and any DNS name servers.")

	grid = Grid (2, 4)
	grid.setField (Label ("Gateway:"), 0, 0, anchorLeft = 1)
	grid.setField (Label ("Primary nameserver:"), 0, 1, anchorLeft = 1)
	grid.setField (Label ("Secondary nameserver:"), 0, 2, anchorLeft = 1)
	grid.setField (Label ("Tertiary nameserver:"), 0, 3, anchorLeft = 1)

	self.gw = Entry (16)
	self.ns = Entry (16)
        self.calcGW()
	self.ns2 = Entry (16)
	self.ns3 = Entry (16)

	if network.gateway:
	    self.gw.set (network.gateway)

	if network.primaryNS:
	    self.ns.set (network.primaryNS)

	self.ns2.set (network.secondaryNS)
	self.ns3.set (network.ternaryNS)

	grid.setField (self.gw,  1, 0, (1, 0, 0, 0))
	grid.setField (self.ns,  1, 1, (1, 0, 0, 0))
	grid.setField (self.ns2, 1, 2, (1, 0, 0, 0))
	grid.setField (self.ns3, 1, 3, (1, 0, 0, 0))

	buttons = ButtonBar (screen, (("OK", "ok"), ("Back", "back")))

	toplevel = GridForm (screen, "Network configuration", 1, 3)
	toplevel.add (text, 0, 0, (0, 0, 0, 1), anchorLeft = 1)
	toplevel.add (grid, 0, 1, (0, 0, 0, 1))
	toplevel.add (buttons, 0, 2, growx = 1)

	result = toplevel.run()
	if buttons.buttonPressed (result) == "ok":
	    retval = INSTALL_OK
	    network.gateway     = self.gw.value()
	    network.primaryNS   = self.ns.value()
	    network.secondaryNS = self.ns2.value()
	    network.ternaryNS   = self.ns3.value()
	else:
	    retval = INSTALL_BACK

	screen.popWindow()
	return retval


    def setGatewayDevice(self, screen, network, devices):
        # See if gateway is located directly on any of the devices

	list = devices.keys()
	list.sort()

        if network.gateway:
            gateway = isys.inet_aton(network.gateway)
            bestNetwork = 0L
            bestNetmask = 0L
            bestDevice = ""

            i = 0
            while i < self.numdevices:
                if devices[list[i]].get("IPADDR") and devices[list[i]].get("NETMASK"):
                    netmask = isys.inet_aton(devices[list[i]].get("NETMASK"))
                    netw = isys.inet_aton(devices[list[i]].get("IPADDR")) & netmask

                    if (gateway & netmask) == netw \
                       and netw >= bestNetwork \
                       and netmask >= bestNetmask:
                        bestNetwork = netw
                        bestNetmask = netmask
                        bestDevice = list[i]

                i = i + 1

            if bestDevice:
                network.gatewayDevice = bestDevice
                return INSTALL_OK
            else:
                ButtonChoiceWindow(screen, "Gateway Device Not Found",
                                   "The gateway is not accessible from any "
                                   "of your network devices. Please make "
                                   "sure your device configurations and "
                                   "gateway is correct.",
                                   buttons = [ 'Back' ])
                return INSTALL_BACK






    def __call__(self, screen, network):
        self.gw = None
        self.ns = None

        devices = network.available ()
        if not devices:
            return INSTALL_NOOP

        if network.readData:
            # XXX expert mode, allow changing network settings here
            return INSTALL_NOOP
        
	list = devices.keys ()
	list.sort()

        self.numdevices = len (list)

	i = 0

# counter for dhcp devices
        cntdhcpdev = 0
	while 1:
	    rc = self.configureDevice (screen, network, devices[list[i]])
	    if rc == INSTALL_BACK:
		i = i-1
		if i < 0:
		    return INSTALL_BACK
	    else:

# is this a dhcp device?
		if devices[list[i]].get("bootproto") == "dhcp":
		    cntdhcpdev = cntdhcpdev+1

		i = i+1

	    if i == len(list):

# if there are only dhcp devices, then we are ready
		if (len(list) - cntdhcpdev) == 0:
		    return INSTALL_OK

		rc = self.configureNetwork(screen, network)
		if rc == INSTALL_BACK:
		    i = i - 1
		else:
                    rc = self.setGatewayDevice(screen, network, devices)
                    if rc == INSTALL_BACK:
			i = 0 # start all over again.
		    else:
                        return INSTALL_OK



class HostnameWindow:
    def __call__(self, screen, network):
        devices = network.available ()
        if not devices:
            return INSTALL_NOOP

        entry = Entry (24)

        if network.hostname != "localhost.localdomain":
            entry.set (network.hostname)

        rc, values = EntryWindow(screen, _("Hostname Configuration"),
             _("The hostname is the name of your computer.  If your "
               "computer is attached to a network, this may be "
               "assigned by your network administrator.\n\n"
               "The hostname should be a fully qualified domain name.\n"
               "Example: myhost.mydomain.topleveldomain"),
             [(_("Hostname"), entry)], buttons = [ TEXT_OK_BUTTON, TEXT_BACK_BUTTON],
	     help = "hostname")

        if rc == TEXT_BACK_CHECK:
            return INSTALL_BACK

        network.hostname = entry.value ()

        return INSTALL_OK
